/*
 * Copyright (C) 2009 Chase Douglas
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as published
 * by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * In addition, as a special exception, the copyright holders give
 * permission to link the code of portions of this program with the
 * OpenSSL library under certain conditions as described in each
 * individual source file, and distribute linked combinations
 * including the two.
 * You must obey the GNU General Public License in all respects
 * for all of the code used other than OpenSSL.  If you modify
 * file(s) with this exception, you may extend this exception to your
 * version of the file(s), but you are not obligated to do so.  If you
 * do not wish to do so, delete this exception statement from your
 * version.
 */

#ifndef SERVER_H
#define SERVER_H

#include <QSslConfiguration>
#include <QTcpServer>

#ifndef __APPLE__
#include <sasl/sasl.h>
#endif

#include "config.h"

#ifdef __APPLE__
#include "MacBroadcaster.h"
#elif defined USE_AVAHI
#include "AvahiBroadcaster.h"
#endif

class QSslCertificate;
class QSslKey;

class Connection;

class Server : public QTcpServer {
    Q_OBJECT

public:
    Server(quint16 _port, QString &_realm, QSslCertificate &cert, QSslKey &key, bool broadcast);

private slots:
    void connectionEncrypted();
    void connectionAuthenticated();
    void checkPass(const QByteArray &user, const QByteArray &pass);

private:
    void incomingConnection(int sock);

    QSslConfiguration config;
    
#ifdef __APPLE__
    MacBroadcaster *broadcaster;
#elif defined USE_AVAHI
    AvahiBroadcaster *broadcaster;
#endif
    
#ifndef __APPLE__
    sasl_conn_t *sasl;
#endif
};

#endif // SERVER_H
